import bpy
import os, sys, math
from glob import glob

if __name__ == "__main__":
    # Read command line arguments
    argv = sys.argv
    argv = argv[argv.index("--") + 1:]  # get all args after "--"
    ply_files = get_list_of_ply_files(ply_dir=argv[0])
    render_dir = argv[1]
    angle = float(argv[2])

    # Stuff that is common for all the images
    delete_everything()
    setup_scene()
    create_camera()
    create_light()
    material = create_material()

    for ply_file in ply_files:
        fluid = create_fluid(ply_file, material, angle)
        image_path = os.path.join(render_dir, fluid.name + '.png')
        render_scene(image_path)
        delete_fluid(fluid)


def get_list_of_ply_files(ply_dir):
    query = os.path.join(ply_dir, '*.ply')
    ply_files = glob(query)
    return sorted(ply_files)


def delete_everything():
    bpy.ops.object.select_all(action='SELECT')
    bpy.ops.object.delete(use_global=False)
    for m in bpy.data.materials:
        bpy.data.materials.remove(m)
    for c in bpy.data.cameras:
        bpy.data.cameras.remove(c)
    for l in bpy.data.lights:
        bpy.data.lights.remove(l)


def setup_scene():
    # Luxcore engine options
    bpy.context.scene.render.engine = 'LUXCORE'
    bpy.context.scene.luxcore.config.device = 'OCL'
    bpy.context.scene.luxcore.denoiser.enabled = True
    bpy.context.scene.luxcore.opencl.use_native_cpu = False

    # Rendering stop criterium (halt condition)
    bpy.context.scene.luxcore.halt.enable = True
    bpy.context.scene.luxcore.halt.use_samples = True
    bpy.context.scene.luxcore.halt.samples = 1000

    # Output image properties
    x_res = 150
    bpy.context.scene.render.resolution_x = x_res
    bpy.context.scene.render.resolution_y = int(x_res * 5.5)
    bpy.context.scene.render.image_settings.color_mode = 'RGBA'

    # Turn-off world's natural light
    bpy.context.scene.world.luxcore.light = 'none'

    # Setup compositor
    bpy.context.scene.use_nodes = True
    render_layers = bpy.context.scene.node_tree.nodes['Render Layers']
    composite = bpy.context.scene.node_tree.nodes['Composite']
    links = bpy.context.scene.node_tree.links
    links.remove(links[0])
    links.new(render_layers.outputs['Alpha'], composite.inputs['Alpha'])
    links.new(render_layers.outputs['DENOISED'], composite.inputs['Image'])


def create_light():
    bpy.ops.object.light_add(
        type='AREA',
        location=(-0.01, 0, 0),
        rotation=(0, -math.pi/2., 0.),
        )
    backlight = bpy.context.active_object
    backlight.name = 'Backlight'
    backlight.data.luxcore.is_laser = True


def create_camera():
    bpy.ops.object.camera_add(
        location=(.02, 0., 2.75E-3),
        rotation=(math.pi/2., 0., math.pi/2.),
        )
    camera = bpy.context.active_object
    camera.data.type = 'ORTHO'
    camera.data.ortho_scale = 0.0055
    camera.data.luxcore.use_clipping = False
    camera.data.luxcore.imagepipeline.transparent_film = True
    bpy.context.scene.camera = camera


def create_material():
    material = bpy.data.materials.new(name="water")
    material.luxcore.node_tree = bpy.data.node_groups.new(
        name="water_node_tree",
        type="luxcore_material_nodes",
        )
    material.luxcore.node_tree.fake_user = True
    nodes = material.luxcore.node_tree.nodes

    mat_output = nodes.new("LuxCoreNodeMatOutput")
    mat_output.location = (200, 0)

    mat_specs = nodes.new("LuxCoreNodeMatGlass")
    mat_specs.location = (0, 0)
    mat_specs.rough = True
    mat_specs.inputs["Roughness"].default_value = 0.02
    mat_specs.inputs["IOR"].default_value = 1.33
    mat_specs.inputs["Opacity"].default_value = 1.0

    material.luxcore.node_tree.links.new(
        mat_specs.outputs["Material"],
        mat_output.inputs["Material"])

    return material


def create_fluid(ply_file_path, material, angle, smooth=True):
    # Import ply geometry
    bpy.ops.import_mesh.ply(filepath=ply_file_path)
    fluid = bpy.context.active_object
    fluid.rotation_euler[2] = angle

    if smooth:
        # Corrective smooth modifier (tries to keep volume constant)
        fluid.modifiers.new(name="smoother", type='CORRECTIVE_SMOOTH')
        fluid.modifiers["smoother"].factor = 0.5
        fluid.modifiers["smoother"].iterations = 30
        fluid.modifiers["smoother"].smooth_type = 'LENGTH_WEIGHTED'
        fluid.modifiers["smoother"].use_only_smooth = True
        fluid.modifiers["smoother"].use_pin_boundary = True
        bpy.ops.object.modifier_apply(modifier="smoother")
        # Smooth shader
        bpy.ops.object.shade_smooth()

    # Apply water material
    fluid.data.materials.append(material)

    return fluid

def delete_fluid(fluid):
    bpy.ops.object.select_all(action='DESELECT')
    fluid.select_set(True)
    bpy.ops.object.delete()

def render_scene(file_name):
    bpy.context.scene.render.filepath = file_name
    bpy.ops.render.render(write_still=True)


